/****************************************************************************
 * UniPROG Universal programmer 
 * Frohwein-Sshua Flash Programmer support
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 * original code by Jeff Frohwein
 *
 * 20/05/2001 - First version, Test hardware
 * 06/06/2001 - Added eepWriteByte, is more generic
 *              eepTestHardware don't work now
 * 07/09/2003 - Ported to new multiprogrammer architecture
 *              eepTestHardware works now
 */

#define EEPROG_C

#include "stdhdr.h"
#include "progdev.h"
#include "portio.h"
#include "eeprog.h"

// Helpfull macros

#define uSleep(delay)   for(slp=0; slp<(10*delay); slp++);
int slp;

#define OUTDATA()       {OUTPORT(eepDataPort, eepData);} // uSleep(10);}
#define OUTCTRL()       {OUTPORT(eepCtrlPort, eepControl);} // uSleep(10);}


// Private variables

BYTE     eepData;
BYTE     eepControl;
DWORD    eepAddress;
int      eepWaitDelay;
unsigned eepCtrlPort;
unsigned eepDataPort;
unsigned eepStatusPort;

/* Programmer device structure */
struct prog_dev pd_eep;

// Basic I/O Functions
int eepInit ( void )   {
  struct prog_dev *pd = &pd_eep;

  eepData = 0x00;
  eepControl = 0xc;
  eepDataPort = 0x378;
  eepStatusPort = eepDataPort + 1;
  eepCtrlPort = eepDataPort + 2;
  eepAddress = 0L;

  memset(pd, 0, sizeof(*pd));
  strcpy(pd->name, "eep");
  strcpy(pd->description, "Frohwein-Sshua Flash Programmer");
  pd->open = eepOpen;
  pd->close = eepClose;
  pd->read_byte_addr = eepReadByteAddr;
  pd->write_byte_addr = eepWriteByteAddr;
  pd->test_hardware = eepCheckHard;
  
  /* register device */
  prgdev_register( &pd_eep );

  return 1;
}

int eepOpen(struct prog_dev *pd ) {

#ifdef __LINUX__
   if (ioperm(eepDataPort, 3, 1) == -1) {
   	printf("Error, i/o access denied for spp/epp\n");
   	return FALSE;
   	}
                                                                                                                             
   printf("Usermode i/o access granted\n");
#endif

   pd->refs++;                                                                                                                            
   return TRUE;
   }


int eepClose( struct prog_dev *pd )
   {
   eepData = 0x00; OUTCTRL();
   eepControl = 0xc; OUTDATA();

   pd->refs--;
   return 1;
   }



// Read a byte from Chip current address
unsigned char eepReadByteAddr( struct prog_dev *pd, long addr )
   {
   unsigned char i, i2;

   eepSetAddr(addr); // Sets Addr

   eepControl |= 2; OUTCTRL();                 /* Set /RD low */
   eepData &=~0x80; OUTDATA();                 /* Set U/L low */
   i = (INPORT(eepStatusPort) >> 3) & 0xf;
   eepData |= 0x80; OUTDATA();                 /* Set U/L high */
   i2= ((INPORT(eepStatusPort) << 1) & 0xf0);
   eepControl &=~2; OUTCTRL();                 /* Set /RD high */

   return(i | i2);
   }

// Write a byte to Chip specified address
void eepWriteByteAddr(struct prog_dev *pd, long addr, unsigned char val)
   {
   unsigned char i, Vtmp;
   unsigned char Ser0, Ser1, Ser2;
   Vtmp = val;

   Ser0 =  addr & 0xff;
   Ser1 = (addr >> 8) & 0xff;
   Ser2 = (addr >> 16) & 0xff;

   for(i=0; i<8; i++)
      {
      (Vtmp & 0x80)?(eepData |= 0x08):(eepData &=~0x08);  // Data
      (Ser0 & 0x80)?(eepData |= 0x01):(eepData &=~0x01);  // Addr Lo
      (Ser1 & 0x80)?(eepData |= 0x02):(eepData &=~0x02);  // Addr Mi
      (Ser2 & 0x80)?(eepData |= 0x04):(eepData &=~0x04);  // Addr Hi

      OUTPORT(eepDataPort, eepData);
      eepData |= (0x10 | 0x40); OUTDATA(); /* Clock data into data & addr SR */
      eepData &=~(0x10 | 0x40); OUTDATA();

      Vtmp <<= 1;       // next bit
      Ser0 <<= 1;
      Ser1 <<= 1;
      Ser2 <<= 1;
      }

   eepData |= (0x10 | 0x40); OUTDATA(); /* Clock data into data & addr latch */
   eepData &=~(0x10 | 0x40); OUTDATA();

   eepAddress = addr;   // addr Changed

   eepControl &=~4; OUTCTRL();                 /* Set /WOE low */
   eepControl |= 1; OUTCTRL();                 /* Set /WR low */
   eepControl &=~1; OUTCTRL();                 /* Set /WR high */
   eepControl |= 4; OUTCTRL();                 /* Set /WOE high */
   }



void SetWE ( int value )
  {
  if (!value)
   { eepControl |= 1; OUTCTRL() }                  /* Set /WR low */
  else
   { eepControl &=~1; OUTCTRL();}                 /* Set /WR high */
  }

void SetRD ( int value )
  {
  if (!value)
   { eepControl |= 2; OUTCTRL();}                  /* Set /RD low */
  else
   { eepControl &=~2; OUTCTRL();}                 /* Set /RD high */
  }

void SetWOE ( int value )
  {
  if (!value)
   { eepControl &=~4; OUTCTRL();}                  /* Set /WOE low */
  else
   { eepControl |= 4; OUTCTRL();}                 /* Set /WOE high */
  }


// Read a byte from Chip current address
BYTE eepReadByte( void )
   {
   BYTE i, i2;

   eepControl |= 2; OUTCTRL();                 /* Set /RD low */
   eepData &=~0x80; OUTDATA();                 /* Set U/L low */
   i = (INPORT(eepStatusPort) >> 3) & 0xf;
   eepData |= 0x80; OUTDATA();                 /* Set U/L high */
   i2 = ((INPORT(eepStatusPort) << 1) & 0xf0);
   eepControl &=~2; OUTCTRL();                 /* Set /RD high */

   return(i | i2);
   }


// Write a byte to Chip current address
void eepWriteByte(BYTE val)
   {
   BYTE i, Vtmp;
   Vtmp = val;

   for(i=0; i<8; i++)
      {
      (Vtmp & 0x80)?(eepData |= 0x08):(eepData &=~0x08);
      OUTDATA();
      eepData |= 0x10; OUTDATA(); /* Clock data into SR */
      eepData &=~0x10; OUTDATA();
	  Vtmp <<= 1;
      }

   eepData |= 0x10; OUTDATA(); /* Clock data into latch */
   eepData &=~0x10; OUTDATA();

   eepControl &=~4; OUTCTRL();                 /* Set /WOE low */
   eepControl |= 1; OUTCTRL();                 /* Set /WR low */
   eepControl &=~1; OUTCTRL();                 /* Set /WR high */
   eepControl |= 4; OUTCTRL();                 /* Set /WOE high */
   }


/* Set chip address */

void eepSetAddr (long Addr)
   {
   BYTE Ser0, Ser1, Ser2, i;

   Ser0 =  Addr & 0xff;
   Ser1 = (Addr >> 8) & 0xff;
   Ser2 = (Addr >> 16) & 0xff;

   for(i=0; i<8; i++)
      {
	  (Ser0 & 0x80)?(eepData |= 0x01):(eepData &=~0x01);
	  (Ser1 & 0x80)?(eepData |= 0x02):(eepData &=~0x02);
	  (Ser2 & 0x80)?(eepData |= 0x04):(eepData &=~0x04);

      OUTPORT(eepDataPort, eepData);
      eepData |= 0x40; OUTDATA(); /* Clock addr into SR */
      eepData &=~0x40; OUTDATA();
      Ser0 <<= 1;
      Ser1 <<= 1;
      Ser2 <<= 1;
      }

   eepData |= 0x40; OUTDATA(); /* Clock addr into latch */
   eepData &=~0x40; OUTDATA();

   eepAddress = Addr;
   }

/* Increment chip address */

void eepIncAddr( void )
   {
   eepAddress++;
   eepSetAddr(eepAddress);
   }

long eepGetAddr( void )
   {
   return eepAddress;
   }

// Check Hardware

int eepCheckHard( struct prog_dev *pd, int mode )
   {
   int i;
   int r = TRUE;
   BYTE b1, b2;

   printf("Cheching hardware \n");
   for (i=0; i<256; i++) {
     b1 = (i & 0xFF);
     eepWriteByte(b1);
     eepControl &=~4; OUTCTRL(); /* Set /WOE low */
     b2 = eepReadByte();
     eepControl |=4; OUTCTRL(); /* Set  /WOE high */

     if (b1 != b2)
       {
       printf("%03d = %03d - %s \n", b1, b2, "ERROR");
       r = FALSE;
       }
     }
   if (!r) return r;

   printf("OK\n");

   return r;
   }


void eepReadBlock(unsigned char *pBlock, long blockSize)
   {
   DWORD i = blockSize;
   assert(pBlock);

   while (i--)
      {
      *pBlock++ = eepReadByte();
      eepIncAddr();
      }
   }
